@@ -14,6 +14,11 @@ from pytorch_lightning.loggers import WandbLogger
 from pytorch_lightning.callbacks.early_stopping import EarlyStopping
 from pytorch_lightning import LightningDataModule
 from pytorch_lightning import Trainer
+from getpass import getpass
+from huggingface_hub import HfApi, Repository
+from pathlib import Path
+
+
 
 torch.cuda.empty_cache()
 pl.seed_everything(42)
@@ -87,7 +92,8 @@ class DataModule(Dataset):
 class PLDataModule(LightningDataModule):
     def __init__(
             self,
-            data_df: pd.DataFrame,
+            train_df:pd.DataFrame,
+            test_df:pd.DataFrame,
             tokenizer: T5Tokenizer,
             source_max_token_len: int = 512,
             target_max_token_len: int = 512,
@@ -104,15 +110,14 @@ class PLDataModule(LightningDataModule):
         :param split:
         """
         super().__init__()
-        self.data_df = data_df
+        self.train_df = train_df
+        self.test_df = test_df
         self.split = split
         self.batch_size = batch_size
         self.target_max_token_len = target_max_token_len
         self.source_max_token_len = source_max_token_len
         self.tokenizer = tokenizer
 
-    def prepare_data(self):
-        self.train_df, self.test_df = train_test_split(self.data_df, test_size=self.split, shuffle=True)
 
     def setup(self, stage=None):
         self.train_dataset = DataModule(
@@ -131,19 +136,19 @@ class PLDataModule(LightningDataModule):
     def train_dataloader(self):
         """ training dataloader """
         return DataLoader(
-            self.train_dataset, batch_size=self.batch_size, shuffle=True, num_workers=8
+            self.train_dataset, batch_size=self.batch_size, shuffle=True, num_workers=2
         )
 
     def test_dataloader(self):
         """ test dataloader """
         return DataLoader(
-            self.test_dataset, batch_size=self.batch_size, shuffle=False, num_workers=8
+            self.test_dataset, batch_size=self.batch_size, shuffle=False, num_workers=2
         )
 
     def val_dataloader(self):
         """ validation dataloader """
         return DataLoader(
-            self.test_dataset, batch_size=self.batch_size, shuffle=False, num_workers=8
+            self.test_dataset, batch_size=self.batch_size, shuffle=False, num_workers=2
         )
 
 
@@ -204,8 +209,9 @@ class LightningModel(pl.LightningModule):
             decoder_attention_mask=labels_attention_mask,
             labels=labels,
         )
-
+        acc = self.val_acc(outputs.logits.argmax(1), labels)
         self.log("val_loss", loss, prog_bar=True, logger=True)
+        self.log(f"val_acc", acc, prog_bar=True,logger=True)
         return loss
 
     def test_step(self, batch, batch_size):
@@ -252,7 +258,8 @@ class trainer:
 
     def train(
             self,
-            data_df: pd.DataFrame,
+            train_df: pd.DataFrame,
+            test_df: pd.DataFrame,
             source_max_token_len: int = 512,
             target_max_token_len: int = 512,
             batch_size: int = 8,
@@ -273,12 +280,17 @@ class trainer:
             use_gpu (bool, optional): if True, model uses gpu for training. Defaults to True.
             outputdir (str, optional): output directory to save model checkpoints. Defaults to "outputs".
             early_stopping_patience_epochs (int, optional): monitors val_loss on epoch end and stops training, if val_loss does not improve after the specied number of epochs. set 0 to disable early stopping. Defaults to 0 (disabled)
+            :param test_df:
+            :param train_df:
         """
         self.target_max_token_len = target_max_token_len
         self.max_epoch = max_epochs
+        self.train_df = train_df
+        self.test_df = test_df
 
         self.data_module = PLDataModule(
-            data_df=data_df,
+            train_df=train_df,
+            test_df= test_df,
             tokenizer=self.tokenizer,
             batch_size=batch_size,
             source_max_token_len=source_max_token_len,
@@ -342,14 +354,14 @@ class trainer:
 
     def save_model(
             self,
-            model_dir="outputs"
+            model_dir="model"
     ):
         """
         Save model to dir
         :param model_dir:
         :return: model is saved
         """
-        path = f"{model_dir}/keytotext-epoch-{self.max_epoch}"
+        path = f"{model_dir}"
         self.tokenizer.save_pretrained(path)
         self.model.save_pretrained(path)
 
@@ -423,3 +435,71 @@ class trainer:
             for g in generated_ids
         ]
         return preds[0]
+
+    def upload(self,model_name,hf_username):
+        hf_password = getpass("Enter your HuggingFace password")
+        token = HfApi().login(username=hf_username, password=hf_password)
+        del hf_password
+        model_url = HfApi().create_repo(token=token, name=model_name, exist_ok=True)
+        model_repo = Repository("./model", clone_from=model_url, use_auth_token=token,
+                                git_email=f"{hf_username}@users.noreply.huggingface.co", git_user=hf_username)
+
+        readme_txt= f"""
+        ---
+language: "en"
+thumbnail: "Keywords to Sentences"
+tags:
+- keytotext
+- k2t
+- Keywords to Sentences
+license: "MIT"
+datasets:
+- WebNLG
+- Dart 
+metrics:
+- NLG
+
+model-index:
+- name: {model_name}
+---
+        
+        <h1 align="center">keytotext</h1>
+
+[![pypi Version](https://img.shields.io/pypi/v/keytotext.svg?logo=pypi&logoColor=white)](https://pypi.org/project/keytotext/)
+[![Downloads](https://static.pepy.tech/personalized-badge/keytotext?period=total&units=none&left_color=grey&right_color=orange&left_text=Pip%20Downloads)](https://pepy.tech/project/keytotext)
+[![Open In Colab](https://colab.research.google.com/assets/colab-badge.svg)](https://colab.research.google.com/github/gagan3012/keytotext/blob/master/notebooks/K2T.ipynb)
+[![Streamlit App](https://static.streamlit.io/badges/streamlit_badge_black_white.svg)](https://share.streamlit.io/gagan3012/keytotext/UI/app.py)
+[![API Call](https://img.shields.io/badge/-FastAPI-red?logo=fastapi&labelColor=white)](https://github.com/gagan3012/keytotext#api)
+[![Docker Call](https://img.shields.io/badge/-Docker%20Image-blue?logo=docker&labelColor=white)](https://hub.docker.com/r/gagan30/keytotext)
+[![HuggingFace](https://img.shields.io/badge/%F0%9F%A4%97-Models%20on%20Hub-yellow)](https://huggingface.co/models?filter=keytotext)
+[![Documentation Status](https://readthedocs.org/projects/keytotext/badge/?version=latest)](https://keytotext.readthedocs.io/en/latest/?badge=latest)
+[![Code style: black](https://img.shields.io/badge/code%20style-black-000000.svg)](https://github.com/psf/black)
+
+
+
+![keytotext](https://socialify.git.ci/gagan3012/keytotext/image?description=1&forks=1&language=1&owner=1&stargazers=1&theme=Light)
+
+
+Idea is to build a model which will take keywords as inputs and generate sentences as outputs.
+
+Potential use case can include: 
+- Marketing 
+- Search Engine Optimization
+- Topic generation etc.
+- Fine tuning of topic modeling models 
+        """.strip()
+
+        (Path(model_repo.local_dir) / 'README.md').write_text(readme_txt)
+        commit_url = model_repo.push_to_hub()
+
+        print("Check out your model at:")
+        print(f"https://huggingface.co/{hf_username}/{model_name}")
+
+
+
+
+
+
+
+
+
